@@ -16,7 +16,6 @@ Cu.import("resource://unity/unity-favicon-utils.js");
 Cu.import("resource://unity/unity-misc-utils.js");
 Cu.import("resource://unity/unity-popup-manager.js");
 Cu.import("resource://unity/unity-preview-utils.js");
-Cu.import("resource://unity/unity-view-manager.js");
 Cu.import("resource://unity/unity-webapps.js");
 Cu.import("resource://unity/unity-window-helper.js");
 Cu.import("resource://unity/unity-xid-helper.js");
@@ -128,63 +127,8 @@ function requestIntegration(uwa, aWindow, domain, name, iconUrl, realInit) {
 				    dontAskActionCallback);
 }
 
-function BrandingRemover(mainWindow) {
-    this._init(mainWindow);
-}
-
-BrandingRemover.prototype = {
-    _value: '',
-
-    _tabs: [],
-
-    _init: function (mainWindow) {
-	this._mainWindow = mainWindow;
-    },
-
-    _func: function (id, oldValue, newValue) {
-	this._value = newValue;
-	if (this._mainWindow.gBrowser.contentWindow.document.title)
-	    return this._mainWindow.gBrowser.contentWindow.document.title;
-	return newValue;
-    },
-
-    activate: function(aWindow) {
-	if (this._tabs.indexOf(aWindow) == -1)
-	    this._tabs.push(aWindow);
-	if (this._tabs.length == 1) {
-	    this._mainWindow.document.watch('title', this._func.bind(this));
-	    this._mainWindow.document.title = this._mainWindow.document.title;
-	}
-    },
-
-    deactivate: function(aWindow) {
-	this._tabs = this._tabs.filter(function (element, index, array) {
-            return element != aWindow;
-        });
-	if (!this._tabs.length) {
-	    this._mainWindow.document.unwatch('title');
-	    if (this._value)
-		this._mainWindow.document.title = this._value;
-	    this._value = undefined;
-	}
-    }
-};
-
-var _windows = new WeakMap();
-
-function getBrandingRemover(mainWindow) {
-    var res = _windows.get(mainWindow);
-
-    if (!res) {
-	res = new BrandingRemover(mainWindow);
-	_windows.set(mainWindow, res);
-    }
-
-    return res;
-}
-
-function UnityObject (uwa, service, ContextManager, ViewManager, aWindow) {
-    this._init(uwa, service, ContextManager, ViewManager, aWindow);
+function UnityObject (uwa, service, ContextManager, aWindow) {
+    this._init(uwa, service, ContextManager, aWindow);
 }
 
 UnityObject.prototype = {
@@ -192,24 +136,22 @@ UnityObject.prototype = {
     uwa: null,
     service: null,
     _contextManager: null,
-    _viewManager: null,
 
     _aWindow: null,
-    _brandingRemover: null,
     _mainWindow: null,
     _windowHelper: null,
     _appRaiseCb: null,
     _appCloseCb: null,
+    _active: false,
 
     _callbackManager: null,
 
-    _init: function (uwa, service, ContextManager, ViewManager, aWindow) {
+    _init: function (uwa, service, ContextManager, aWindow) {
         this._events = [];
         this._unity = { context: null, onInitCallbacks: [], self: this };
         this.uwa = uwa;
         this.service = service;
         this._contextManager = ContextManager;
-        this._viewManager = ViewManager;
 
 	this._callbackManager = new UnityCallbackManager();
 
@@ -308,19 +250,9 @@ UnityObject.prototype = {
 	    }
 	}
 
-	var toplevelWindow = this._windowHelper.FindToplevelContentWindow (this._aWindow);
-	if (this._viewManager.windowIsSelected (this._aWindow, toplevelWindow)) {
-	    this._viewManager.setActiveView (this._unity.context, toplevelWindow);
-	} else {
-	    this.uwa.context_set_view_is_active (this._unity.context, 0);
-	}
-
 	this.uwa.context_set_view_location (this._unity.context, this._aWindow.document.location.toString());
-	XIDHelper.setWindowXID(this._aWindow, this.uwa, this._unity.context);
 
-	this._listenEvent(this._mainWindow.gBrowser.tabContainer, "TabSelect", this._onTabSelect);
-
-	this._onTabSelect();
+	this.setActive(this._active);
     },
 
     _realInit: function(name, iconUrl, firstTime) {
@@ -395,24 +327,15 @@ UnityObject.prototype = {
 	    .rootTreeItem
 	    .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
             .getInterface(Components.interfaces.nsIDOMWindow);
-        this._brandingRemover = getBrandingRemover(this._mainWindow);
 
-        this._listenEvent(this._mainWindow.gBrowser.tabContainer, "TabClose", function(event) {
-            if (this._mainWindow.gBrowser.getBrowserForTab(event.target) == this._mainWindow.gBrowser.getBrowserForDocument(
-                this._windowHelper.FindToplevelContentWindow(aWindow).document)) {
-
-            	this._loseInterest(true);
-            }
-        });
         this._listenEvent(aWindow, "pagehide", function (event) {
-            this._loseInterest(false);
+            this._loseInterest(true);
         });
 
 	this._appRaiseCb = this._callbackManager.makeCallback (this.uwa.ContextRaiseCallbackType,
 							       this._windowHelper.makeRaiseCallback(this._unity), "raise");
 	this._appCloseCb = this._callbackManager.makeCallback (this.uwa.ContextCloseCallbackType,
 							       this._windowHelper.makeCloseCallback(), "close");
-
     },
 
     _previewRequested: function (context, user_data) {
@@ -425,17 +348,18 @@ UnityObject.prototype = {
 	return ctypes.char.array(previewData.length+1)(previewData+'\0');
     },
 
-    _onTabSelect: function(event) {
-	let window = this._mainWindow.gBrowser.contentWindow;
-	let context = this._contextManager.findContextByWindow(window);
+    hasContext: function() {
+        return this._unity.context != null;
+    },
 
-	this._viewManager.setActiveView (context, window);
+    setActive: function(value) {
+        if (this._unity.context) {
+	    XIDHelper.setWindowXID(this._aWindow, this.uwa, this._unity.context);
 
-	if (window != this._aWindow) {
-	    this._brandingRemover.deactivate(this._aWindow);
-	} else {
-	    this._brandingRemover.activate(this._aWindow);
-	}
+            this.uwa.context_set_view_is_active (this._unity.context, value);
+        }
+
+        this._active = value;
     },
 
     _loseInterest: function (abandoned) {
@@ -444,7 +368,6 @@ UnityObject.prototype = {
         }
 
         this._unity.context = null;
-        this._brandingRemover.deactivate(this._aWindow);
         this._removeAllListeners();
         this._callbackManager.releaseAll();
     }
@@ -554,7 +477,6 @@ UnityObjectFactory.prototype = {
         this.libnotify = new Libnotify();
         this.service = this.uwa.service_new();
         this.contextManager = new UnityContextManager();
-        this.viewManager = new UnityViewManager(this.contextManager, this.uwa);
 
         this.libnotify.init("firefox");
 
@@ -565,24 +487,31 @@ UnityObjectFactory.prototype = {
         aWindow = FindToplevelContentWindow(aWindow);
 
         if (!this._windows.has(aWindow)) {
-            this._windows.set(aWindow, { unity: new UnityObject(this.uwa, this.service, this.contextManager, this.viewManager, aWindow).getAPI(),
+            this._windows.set(aWindow, { unity: new UnityObject(this.uwa, this.service, this.contextManager, aWindow),
                                          webkitNotifications: new WebkitNotifications(this.uwa, this.libnotify, aWindow) });
         }
         return this._windows.get(aWindow);
     },
 
     getForWindow: function(aWindow) {
-        return this._get(aWindow).unity;
+        return this._get(aWindow).unity.getAPI();
     },
 
     getWebkitNotificationsForWindow: function(aWindow) {
         return this._get(aWindow).webkitNotifications;
+    },
+
+    getExistingForWindow: function(aWindow) {
+        if (!this._windows.has(aWindow))
+            return null;
+        return this._get(aWindow).unity;
     }
 };
 
 var objectFactory = new UnityObjectFactory();
 
 UnityGlobalPropertyInitializer = {
+    _listeners: [],
     handle: function (aWindow) {
         if (!objectFactory.uwa.permissions_is_integration_allowed())
             return;
@@ -602,6 +531,22 @@ UnityGlobalPropertyInitializer = {
         };
         aWindow.wrappedJSObject.external = new unity();
         aWindow.wrappedJSObject.webkitNotifications = objectFactory.getWebkitNotificationsForWindow(aWindow);
+
+        for (i = 0; i < this._listeners.length; i++) {
+            this._listeners[i](aWindow);
+        }
+    },
+
+    connect: function(signalName, listener) {
+        UnityGlobalPropertyInitializer._listeners.push(listener);
+    },
+
+    disconnectAll: function() {
+        this._listeners = [];
+    },
+
+    getExistingForWindow: function(aWindow) {
+        return objectFactory.getExistingForWindow(aWindow);
     }
 };
 
